import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList; // Listen importieren

public class Tier extends Actor {

    /**
     * Methode ermittelt ein freies nicht besetztes Nachbarfeld
     * erstmal prüfen, ob Feld am Rand liegt
     * wenn ja, dann auch prüfen, ob das Feld gegenüber frei ist
     * das für alle freien Nachfarfelder-Koordinaten machen und in 
     * eine Liste speichern
     * 
     * @return freie Koordinate mit X- und Y-Wert
     */
    public Koordinate getZufaelligesFreiesNachbarfeld() {
        // mögliche Kandidaten für neue Koordinaten in Liste aufnehmen
        ArrayList<Koordinate> l = new ArrayList<Koordinate>();

        // links
        if (getX() == 0) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(getWorld().getWidth()-1,0,Actor.class) == null) {
                l.add(new Koordinate(getWorld().getWidth()-1, getY()));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(-1,0,Actor.class) == null) {
                l.add(new Koordinate(getX()-1, getY()));
            }
        }

        // rechts
        if (getX() == getWorld().getWidth()-1) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(-getWorld().getWidth()+1,0,Actor.class) == null) {
                l.add(new Koordinate(0, getY()));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(1,0,Actor.class) == null) {
                l.add(new Koordinate(getX()+1, getY()));
            }
        }

        // oben
        if (getY() == 0) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(0, getWorld().getHeight()-1,Actor.class) == null) {
                l.add(new Koordinate(getX(), getWorld().getHeight()-1));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(0,-1,Actor.class) == null) {
                l.add(new Koordinate(getX(), getY()-1));
            }
        } 

        // unten
        if (getY() == getWorld().getHeight()-1) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(0, -getWorld().getHeight()+1,Actor.class) == null) {
                l.add(new Koordinate(getX(), 0));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(0,1,Actor.class) == null) {
                l.add(new Koordinate(getX(), getY()+1));
            }
        } 

        // falls es überhaupt freie Felder gibt eine zufällige Koordinate returnen
        if (l.size() > 0) {
            int r = Greenfoot.getRandomNumber(l.size());
            return (new Koordinate(l.get(r).getX(), l.get(r).getY()));
        }
        else { // sonst nur null returnen
            return null;
        }
    }

    /**
     * Diese Methode arbeitet ähnlich wie die Methode
     * getZufaelligesFreiesNachbarfeld() mit dem Unterschied, dass
     * hier explizit nach einer Koordinate gesucht wird, auf der
     * sich ein Fisch befindet
     * 
     * @return Koordinate mit X- und Y-Wert
     */
    public Koordinate getNachbarfeldMitFisch() {
        // mögliche Kandidaten für neue Koordinaten
        ArrayList<Koordinate> l = new ArrayList<Koordinate>();
        // erstmal prüfen, ob Feld am Rand liegt
        // wenn ja, dann auch prüfen, ob das Feld gegenüber frei ist
        // so für alle freien Nachfarfelder-Koordinaten machen und in eine Liste speichern

        // links
        if (getX() == 0) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(getWorld().getWidth()-1,0,Fisch.class) != null) {
                l.add(new Koordinate(getWorld().getWidth()-1, getY()));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(-1,0,Fisch.class) != null) {
                l.add(new Koordinate(getX()-1, getY()));
            }
        }

        // rechts
        if (getX() == getWorld().getWidth()-1) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(-getWorld().getWidth()+1,0,Fisch.class) != null) {
                l.add(new Koordinate(0, getY()));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(1,0,Fisch.class) != null) {
                l.add(new Koordinate(getX()+1, getY()));
            }
        }

        // oben
        if (getY() == 0) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(0, getWorld().getHeight()-1,Fisch.class) != null) {
                l.add(new Koordinate(getX(), getWorld().getHeight()-1));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(0,-1,Fisch.class) != null) {
                l.add(new Koordinate(getX(), getY()-1));
            }
        } 

        // unten
        if (getY() == getWorld().getHeight()-1) {
            // Feld gegenüber prüfen
            if (getOneObjectAtOffset(0, -getWorld().getHeight()+1,Fisch.class) != null) {
                l.add(new Koordinate(getX(), 0));
            }
        }
        else {
            // unmittelbares Nachbarfeld prüfen
            if (getOneObjectAtOffset(0,1,Fisch.class) != null) {
                l.add(new Koordinate(getX(), getY()+1));
            }
        } 

        // falls es Felder mit Fischen gibt
        if (l.size() > 0) {
            int r = Greenfoot.getRandomNumber(l.size());
            return (new Koordinate(l.get(r).getX(), l.get(r).getY()));
        }
        else { // sonst nur null returnen
            return null;
        }
    }

    /**
     * Methode erzeugt einen neuen Fisch oder einen neuen Hai
     * auf einem freien Nachbarfeld, falls eines vorhanden ist
     * 
     * @param Fisch- oder Hai-Objekt
     */
    public void erzeugeNachkomme(Tier pTier) {        
        Koordinate fKoord = getZufaelligesFreiesNachbarfeld();
        if (fKoord != null) {
            getWorld().addObject(pTier, fKoord.getX(), fKoord.getY());
        }
    }

}